<?php

namespace FiloBlu\CategoryPagination\Block\Catalog\Product\ProductList;

use Magento\Catalog\Helper\Product\ProductList;
use Magento\Catalog\Model\Product\ProductList\Toolbar as ToolbarModel;

class Toolbar extends \Magento\Catalog\Block\Product\ProductList\Toolbar
{
    protected $request;
    protected $registry;
    protected $helper;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Catalog\Model\Session $catalogSession,
        \Magento\Catalog\Model\Config $catalogConfig,
        ToolbarModel $toolbarModel,
        \Magento\Framework\Url\EncoderInterface $urlEncoder,
        ProductList $productListHelper,
        \Magento\Framework\Data\Helper\PostHelper $postDataHelper,
        array $data = [],
        \Magento\Framework\App\Request\Http $request,
        \Magento\Framework\Registry $registry,
        \FiloBlu\CategoryPagination\Helper\Data $helper
    )
    {
        $this->_catalogSession = $catalogSession;
        $this->_catalogConfig = $catalogConfig;
        $this->_toolbarModel = $toolbarModel;
        $this->urlEncoder = $urlEncoder;
        $this->_productListHelper = $productListHelper;
        $this->_postDataHelper = $postDataHelper;

        $this->request = $request;
        $this->registry = $registry;
        $this->helper = $helper;

        parent::__construct($context, $catalogSession, $catalogConfig, $toolbarModel, $urlEncoder, $productListHelper, $postDataHelper, $data);
    }


    /**
     * Set collection to pager
     *
     * @param \Magento\Framework\Data\Collection $collection
     * @return $this
     */
    public function setCollection($collection)
    {
        if ($this->helper->getCountPaginationMapping()) {
            $this->_collection = $collection;

            // we need to set pagination only if passed value integer and more that 0
            $limit = (int)$this->getLimit();

            if ($limit) {

                $_categoryId = -1;

                // for avoid problems on search
                if ($this->registry->registry("current_category")) {
                    $_categoryId = $this->registry->registry("current_category")->getId();
                }

                $offset = 0;
                $page = (int)$this->request->getParam('p', 1);

                if ($page > 1) {

                    $listMode = $this->getCurrentMode();
                    $productNumberPage1 = $this->helper->getProductNumber($listMode, 1, $_categoryId);
                    $productNumberPageOther = $this->helper->getProductNumber($listMode, 2, $_categoryId);

                    $offset = $productNumberPage1 + ($productNumberPageOther * ($page - 2));

                    if (!$this->getRequest()->isXmlHttpRequest()) {
                        $limit = $offset + $limit;
                        $offset = 0;
                    }
                }

                $this->_collection->getSelect()->limit($limit, $offset);
            }
            if ($this->getCurrentOrder()) {
                $this->_collection->setOrder($this->getCurrentOrder(), $this->getCurrentDirection());
            }


            return $this;
        } else {
            parent::setCollection($collection);
        }
    }

    /**
     * Retrieve available limits for current view mode
     *
     * @return array
     */
    public function getAvailableLimit()
    {
        if ($this->helper->getCountPaginationMapping()) {
            if ($this->getCurrentMode() == "grid") {
                $limit = $this->helper->getPageSizeForCategoryGrid();
            } else {
                $limit = $this->helper->getPageSizeForCategoryList();
            }

            return array($limit => $limit);
        } else {
            return parent::getAvailableLimit();
        }

    }

}
